#include "SoftwareSerial.h"
#include <XBee.h>
SoftwareSerial XBeeSerial(2, 3);
XBee xbee = XBee();

int GREEN_LED = 10;
int ORANGE_LED =9;
int RED_LED =8;

unsigned long lastCallS2 =0;
unsigned long lastCallS3 =0;
unsigned int restartsS2=0;
unsigned int restartsS3=0;
boolean restartFailed=false;
uint16_t lastRestartedDevice=0x0000;
char timeCoor[]={0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00};

void setup()
{
  //porty 0 a 1
  Serial.begin(9600);
  //porty 2 a 3
  XBeeSerial.begin(9600);
  xbee.setSerial(XBeeSerial);
  pinMode(GREEN_LED,OUTPUT);
  pinMode(ORANGE_LED,OUTPUT);
  pinMode(RED_LED,OUTPUT);
}

void loop()
{
  getAndProcessFrame();
  splitTime();
  //lokalni zmereni
  float  tepmerature = rightTemp(0);
  String sign = "";
  if (tepmerature > 0) {sign = "+";}
  if (abs(tepmerature <10)){sign = sign+"0";}
  String tepmeratureS = sign+String(tepmerature);
  sendFrameToPCCoor('L',tepmeratureS, timeCoor);
  //LED signalizace
  ledMonitor(tepmerature);
  //kontrola end node
  checkNodes();
  delay(500);
}
//vypočet průměrné teploty z 10 měření
float rightTemp(int pin) {
  float temperature = 0.0;
  int sample;
  float ten_samples = 0.0;

  for (sample = 0; sample < 10; sample++) {
    temperature = ((float)analogRead(pin) * 5.0 / 1024.0) - 0.5;
    temperature = temperature / 0.01;
    // měření každých 100ms
    delay(50);
    ten_samples = ten_samples + temperature;
  }
  temperature = ten_samples / 10.0;
  return temperature;
}
void checkNodes(){
  unsigned long now = millis();
  if(now-lastCallS2>60000UL){
    sendResetFrame(0x0002);
    lastCallS2=millis();
  }
  if(now-lastCallS3>60000UL){   
    sendResetFrame(0x0003);
    lastCallS3=millis();
  }  
}
void sendResetFrame(uint16_t destination){
  uint8_t payload[] = {0x66};
  //Vytvoření žádosti v Series 1
  lastRestartedDevice=destination;
  Tx16Request tx = Tx16Request(destination, payload, sizeof(payload));
  
  //Odeslání rámce
  xbee.send(tx);
}
void sendFrameToPC(char source, int rssi, String temp, int sendP[], int receivedP[], int timeNode[]){
  Serial.write(0x7E);
  Serial.print(source);
  Serial.write(rssi);
  Serial.write(temp.length());
  Serial.print(temp);
  Serial.write(sendP[0]);
  Serial.write(sendP[1]);
  Serial.write(sendP[2]);
  Serial.write(sendP[3]);
  Serial.write(receivedP[0]);
  Serial.write(receivedP[1]);
  Serial.write(receivedP[2]);
  Serial.write(receivedP[3]);
  for(int i=0;i<8;i++){
    Serial.write(timeNode[i]);  
  }
  if(source=='2'){
    Serial.write(restartsS2);
  }else if(source=='3'){
    Serial.write(restartsS3);
  }
}
void sendFrameToPCCoor(char source, String temp, char timeNode[]){
  Serial.write(0x7E);
  Serial.print(source);
  Serial.write(0x64);
  Serial.write(temp.length());
  Serial.print(temp);
  for(int i=7;i>=0;i--){
    Serial.print(timeNode[i]);  
  }
  if(restartFailed){
    Serial.write(0x01);
    restartFailed=false;
  }else{
    Serial.write(0x00);
  }
  if(lastRestartedDevice==0x0002){
    Serial.print('2');
  }else if(lastRestartedDevice==0x0003){
    Serial.print('3');        
  }
  
}
void ledMonitor(int temp){
  if(temp<25.0){
      digitalWrite(GREEN_LED,HIGH);
      digitalWrite(ORANGE_LED,LOW);
      digitalWrite(RED_LED,LOW);
  }else if(temp<30.0){
    digitalWrite(GREEN_LED,LOW);
      digitalWrite(ORANGE_LED,HIGH);
      digitalWrite(RED_LED,LOW);
  }else{
  digitalWrite(GREEN_LED,LOW);
      digitalWrite(ORANGE_LED,LOW);
      digitalWrite(RED_LED,HIGH);  
  }
}
void splitTime(){
    unsigned long liveTime=millis();
    for(int i=0;i<sizeof(timeCoor);i++){
      if(liveTime>0){
        timeCoor[i] = char(liveTime%0x100UL);
        liveTime=(liveTime/0x100);
      }else{
        timeCoor[i]=0;
      }
    }
}
void getAndProcessFrame(){
  xbee.readPacket(1000);
  Rx16Response rx16 = Rx16Response();
  TxStatusResponse txStatus = TxStatusResponse();  
    if (xbee.getResponse().isAvailable()) {
        if (xbee.getResponse().getApiId() == RX_16_RESPONSE) {
                xbee.getResponse().getRx16Response(rx16);
                processRx16Response(rx16);
        }else if(xbee.getResponse().getApiId() == TX_STATUS_RESPONSE){
                xbee.getResponse().getTxStatusResponse(txStatus);  
                processTxStatus(txStatus);
        }   
    }
    if (xbee.getResponse().isError()) {
    Serial.println("error");
    // get the error code
    int error_code = xbee.getResponse().getErrorCode();
      if(error_code == 1){
        //paket je poškozen
      }
    }
}

void processTxStatus(TxStatusResponse txStatus){
    if(txStatus.isSuccess()){ 
       if(lastRestartedDevice==0x0002){
         restartsS2++;
       }else if(lastRestartedDevice==0x0003){
         restartsS3++;
       }
    }else{
       restartFailed=true;
    }
}

void processRx16Response(Rx16Response rx16){
      
      uint16_t source = rx16.getRemoteAddress16();
      uint8_t rssi = rx16.getRssi();
      char data[6];
      String tempS = "";
      int receivedPackets[]={0x00,0x00,0x00,0x00};
      int sendPackets[]={0x00,0x00,0x00,0x00};
      int timeNode[]={0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00};
      for (int i = 0; i < 6; i++) {
        data[i] = (char)rx16.getData(i);
        tempS += data[i]; 
      }
      for (int i=0;i<4;i++){
        receivedPackets[i]=rx16.getData(i+6);
      }
      for (int i=0;i<4;i++){
        sendPackets[i]=rx16.getData(i+10);
      }
      for (int i=0;i<8;i++){
        timeNode[i]=rx16.getData(i+14);
      }
      char sourceCH=' ';
      if(source==0x0002){
        sourceCH='2';
        lastCallS2=millis();
      }else if(source==0x0003){
        sourceCH='3'; 
        lastCallS3=millis();
      }
      rssi=map(rssi,23,92,0,100);
      rssi=100-rssi;
      sendFrameToPC(sourceCH,rssi,tempS, receivedPackets, sendPackets,timeNode);

      delay(500);  
}
 
